#include <Servo.h>

Servo myservo;

// Setting port numbers for inputs and outputs
const int mos = 9;
const int b1 = 2;
const int b2 = 3;
const int b3 = 4;
const int sensorPin = A0;

// Values corresponding to lengths of notes
int quarter = 500;
int half = quarter*2;
int whole = quarter*4;
int eighth = quarter/2;
int sixteenth = quarter/4;

// Variables holding the values of recorded music
int record_pos[10];
int record_speed[10];
int record_size = 0;

int prev1 = 0;
int prev2 = 0;
int prev3 = 0;

// The current position of the servo motor, initialized to the middle instrument
int cur_pos = 80;

void setup() {
  // put your setup code here, to run once:
  // Setting the mode of each attachment to input or output
  myservo.attach(10);
  myservo.write(80);
  pinMode(mos, OUTPUT);
  pinMode(sensorPin, INPUT);
  pinMode(b1, INPUT);
  pinMode(b2, INPUT);
  pinMode(b3, INPUT);
  Serial.begin(9600);
  delay(50);

  // Instructions for the user
  Serial.println("Welcome to your studio");
  Serial.println("Twist the potentiometer to change the speed of your notes");
  Serial.println("Click the first button to change which sound the machine makes");
  Serial.println("Click the second button to make recordings of sounds");
  Serial.println("Click the third button to play back what you have so far");
}

void loop() {
  // put your main code here, to run repeatedly:

  // Checks if the third button has been pressed
  int b3_val = digitalRead(b3);
  if (b3_val == 0 and prev3 == 1) {
    // Plays back whatever values have been recorded by iterating through the two lists data is stored in
    Serial.println("Playing back your track");
    for (int i = 0; i < record_size; i++) {
      myservo.write(record_pos[i]);
      digitalWrite(mos, HIGH);
      delay(record_speed[i]);
      digitalWrite(mos, LOW);
      delay(record_speed[i]);
    }
    delay(1000);
  }
  prev3 = b3_val;
  

  // Checks if the first button has been pressed
  int b1_val = digitalRead(b1);
  if (b1_val == 0 and prev1 == 1) {
    // Changes the position of the servo motor to point the solenoid towards the next instrument
    if (cur_pos == 80) {
      cur_pos = 40;
    }
    else if (cur_pos == 40) {
      cur_pos = 120;
    }
    else {
      cur_pos = 80;
    }
  }
  prev1 = b1_val;

  myservo.write(cur_pos);

  // Reads the potentiometer value and sets the note speed accordingly
  int pot_val = analogRead(sensorPin);
  int note_type = 0;
  if (pot_val < 170) {
    // Don't play any notes
  }
  else {
    if (pot_val < 341) {
      // Play whole notes
      note_type = whole;
    }
    else if (pot_val < 511) {
      // Play half notes
      note_type = half;
    }
    else if (pot_val < 682) {
      // Play quarter notes
      note_type = quarter;
    }
    else if (pot_val < 852) {
      // Play eighth notes
      note_type = eighth;
    }
    else {
      // Play sixteenth notes
      note_type = sixteenth;
    }

    // Plays a note
    digitalWrite(mos, HIGH);
    delay(note_type);
    digitalWrite(mos, LOW);
    delay(note_type);
    
  }

  // Checks if the second button has been pressed
  int b2_val = digitalRead(b2);
  if (b2_val == 0 and prev2 == 1) {
    // Ensures that the max recording capacity has not been exceeded
    if (record_size < 10) {
      // Records the current position and the current note speed in two separate lists
      Serial.println("Recording a sound");
      record_pos[record_size] = cur_pos;
      record_speed[record_size] = note_type;
      record_size += 1;
    }
    else {
      Serial.println("Maximum recording length reached");
    }
  }
  prev2 = b2_val;
  
}
